/*
 *
 * watchpoint - interface for the watchpoint service
 *
 * Copyright (c) 2016 Rob Gardner <rob.gardner@oracle.com>
 *
 */

#ifndef _SPARC64_WATCHPOINT_H
#define _SPARC64_WATCHPOINT_H

#define MCD_DISABLE_INT
#define MCD_CAS_TRACE 1
#define MCD_LOAD_TRACE 2
#define MCD_STORE_TRACE 4
#define MCD_HV_TRACE 8

#define RDTICK(X) \
	asm volatile(" rd %%tick, %0\n" : "=r" (X))

enum direction {
	load,    /* ld, ldd, ldh, ldsh */
	store,   /* st, std, sth, stsh */
	both,    /* Swap, ldstub, cas, ... */
	fpld,
	fpst,
	invalid,
};

/* a watchpoint incident */
typedef struct {
	enum direction dir;
	unsigned long elapsed_time;
	unsigned long fault_addr;
	unsigned long tpc;
	unsigned long prev_caller;
	int asi, sign, insn, access_size;
	unsigned long rs1_val, rs2_val, rd_val;
	unsigned long old_rd_val;
	unsigned long old_mem_val, new_mem_val;
	int count;
} watchpoint_rec_t;


typedef void (*print_record_t)(watchpoint_rec_t *mr);

typedef struct
{
	print_record_t print_record;
	/* others? */
} watchpoint_services_t;

typedef int (*watchpoint_callback_t)(watchpoint_rec_t *w, watchpoint_services_t *ws, void *private);

#define NUM_WATCHPOINTS 10

/* watchpoint descriptor */
typedef struct
{
	void *private;
	unsigned long addr;
	unsigned long owner;
	watchpoint_callback_t fn;
	int len;
	// stats or other stuff?
} watchpoint_t;


/* internal entry points */
int set_mcde_state(int enable);
void disable_mcde(void);
int watchpoint(struct pt_regs *regs, unsigned long addr, unsigned long context);

/* public entry points */
int register_watchpoint(void *addr, int len, watchpoint_callback_t fn, void *private);
int deregister_watchpoint(void *addr, int len);

/* MCD tag to use */
#define WATCHPOINT_MEMORY_TAG 3


#endif /* ifndef _SPARC64_WATCHPOINT_H */
