/*
 *  Copyright (C) 2001 Andrea Arcangeli <andrea@suse.de> SuSE
 *  Copyright 2003 Andi Kleen, SuSE Labs.
 *
 *  Thanks to hpa@transmeta.com for some useful hint.
 *  Special thanks to Ingo Molnar for his early experience with
 *  a different vsyscall implementation for Linux/IA32 and for the name.
 */

#include <linux/seqlock.h>
#include <linux/time.h>
#include <linux/clocksource.h>

#include <asm/vvar.h>

void update_vsyscall_tz(void)
{
	unsigned long flags;

	if (unlikely(vvar_data == NULL))
		return;

	write_seqlock_irqsave(&vvar_data->gtod.lock, flags);
	/* sys_tz has changed */
	vvar_data->gtod.sys_tz = sys_tz;
	write_sequnlock_irqrestore(&vvar_data->gtod.lock, flags);
}

void update_vsyscall(struct timespec *wall_time, struct timespec *wtm,
			struct clocksource *clock, u32 mult)
{
	unsigned long flags;

	if (unlikely(vvar_data == NULL))
		return;

	write_seqlock_irqsave(&vvar_data->gtod.lock, flags);
	vvar_data->gtod.vclock_mode = clock->archdata.vclock_mode;
	vvar_data->gtod.clock.cycle_last = clock->cycle_last;
	vvar_data->gtod.clock.mask = clock->mask;
	vvar_data->gtod.clock.mult = mult;
	vvar_data->gtod.clock.shift = clock->shift;
	vvar_data->gtod.wall_time_sec = wall_time->tv_sec;
	vvar_data->gtod.wall_time_nsec = wall_time->tv_nsec;
	vvar_data->gtod.wall_to_monotonic = *wtm;
	vvar_data->gtod.wall_time_coarse = __current_kernel_time();
	write_sequnlock_irqrestore(&vvar_data->gtod.lock, flags);
}
