#ifndef _SPARC64_MM_INIT_H
#define _SPARC64_MM_INIT_H

#include <asm/page.h>

/* Most of the symbols in this file are defined in init.c and
 * marked non-static so that assembler code can get at them.
 */

#define MAX_PHYS_ADDRESS	(1UL << MAX_PHYS_ADDRESS_BITS)
#define KPTE_BITMAP_CHUNK_SZ	(256UL * 1024UL * 1024UL)
#define KPTE_BITMAP_BYTES	\
	((MAX_PHYS_ADDRESS / KPTE_BITMAP_CHUNK_SZ) / 4)
#define VALID_ADDR_BITMAP_BYTES	\
	(1UL << (MAX_PHYS_ADDRESS_LOBITS - ILOG2_4MB - 3))

extern unsigned long kern_linear_pte_xor[4];
extern unsigned long kpte_linear_bitmap[KPTE_BITMAP_BYTES / sizeof(unsigned long)];
extern unsigned int sparc64_highest_unlocked_tlb_ent;
extern unsigned long sparc64_kern_pri_context;
extern unsigned long sparc64_kern_pri_nuc_bits;
extern unsigned long sparc64_kern_sec_context;
extern void mmu_info(struct seq_file *m);

struct linux_prom_translation {
	unsigned long virt;
	unsigned long size;
	unsigned long data;
};

/* Exported for kernel TLB miss handling in ktlb.S */
extern struct linux_prom_translation prom_trans[512];
extern unsigned int prom_trans_ents;

/* Exported for SMP bootup purposes. */
extern unsigned long kern_locked_tte_data;

extern void prom_world(int enter);

extern void free_initmem(void);

#ifdef CONFIG_SPARSEMEM_VMEMMAP
#define VMEMMAP_CHUNK_SHIFT	ILOG2_4MB
#define VMEMMAP_CHUNK		(1UL << VMEMMAP_CHUNK_SHIFT)
#define VMEMMAP_CHUNK_MASK	~(VMEMMAP_CHUNK - 1UL)
#define VMEMMAP_ALIGN(x)	(((x)+VMEMMAP_CHUNK-1UL)&VMEMMAP_CHUNK_MASK)

/* This is a region mapped with 4Mb tte-s to cover page structures.
 * 6 is powerof2 of struct page and 3 is powerof2 of tte.
 */
#define	VMEMMAP_SIZE	(1UL << (MAX_PHYS_ADDRESS_LOBITS - PAGE_SHIFT + \
				6 - VMEMMAP_CHUNK_SHIFT + 3))
/* Each one of the entries will be populated with VMEMMAP_SIZE to cover
 * page structures when RAM is present within this MAX_PHYS_ADDRESS_LOBITS
 * region of physical address bits. Otherwise the entry is NULL for a hole.
 */
extern unsigned long vmemmap_table[NR_VMEMMAP_ROOT_ENTRIES];
#endif

#endif /* _SPARC64_MM_INIT_H */
