/*
 * GPL notice goes here
 */

/*
 * Copyright (c) 2014, Oracle and/or its affiliates. All rights reserved.
 */

#ifndef _SYS_DAX_IMPL_H
#define	_SYS_DAX_IMPL_H

#ifdef	__cplusplus
extern "C" {
#endif

#ifndef	_ASM

#include "ccb.h"

typedef struct dax_ctx dax_ctx_t;

typedef struct
{
	struct list_head mm_list;
	struct list_head vma_list;
	struct list_head ctx_list;
	struct mm_struct *this_mm;
	spinlock_t lock;
	int vma_count, ctx_count;
} dax_mm_t;

typedef struct
{
	struct list_head vma_list;
	dax_mm_t *dax_mm;
	struct vm_area_struct *vma;
	void         *kva;	/* kernel virtual address */
	unsigned long sva;	/* synthetic virtual address */
	unsigned long pa;	/* physical address */
	unsigned long tte;	/* TLB entry */
	size_t requested_length;
	size_t actual_length;
} dax_vma_t;

/*
 * Defines for Synthetic Large Pages
 */

#define SVA_BASE_4M   (0x4LL << 50)
#define SVA_BASE_64K  (0x6LL << 50)
#define SVA_BASE_BITS (0xfLL << 50)

/*
 * DAX per thread CCB context structure
 *
 * The member data types are sized such that the structure almost fits
 * within a 64B cacheline.
 */
struct dax_ctx {
	struct task_struct *owner;     /* pointer to thread that owns this ctx */
	struct list_head ctx_list;
	dax_mm_t        *dax_mm;       /* pointer to per process dax mm */
	ccb_t		*ccb_buf;	/* CCB buffer */
	uint64_t	ccb_buf_ra;	/* cached RA */
	struct page     **pages;	/* pages for CCBs */
	ccb_completion_area_t *ca_buf;	/* CCB completion area (CA) buffer */
	uint64_t	ca_buf_ra;	/* cached RA */
	uint32_t	ccb_buflen;	/* bytes */
	uint32_t	ccb_submit_maxlen; /* max user ccb byte len per call */
	uint32_t	ca_buflen;	/* bytes */
	uint32_t	ca_start_off;	/* returned from dax_submit */
	/*
	 * CCB buffer management
	 *
	 * A BIP-Buffer is used to track the outstanding CCBs.
	 *
	 * A BIP-Buffer is a well-known variant of a circular buffer that
	 * returns variable length contiguous blocks.  The buffer is split
	 * into two regions, A and B.  The buffer starts with a single region A.
	 * When there is more space before region A than after, a new region B
	 * is created and future allocations come from region B.  When region A
	 * is completely deallocated, region B if in use is renamed to region A.
	 */
	uint32_t	a_start;
	uint32_t	a_end;
	uint32_t	b_end;		/* region B always starts at 0 */
	uint32_t	resv_start;	/* current reservation */
	uint32_t	resv_end;
	uint32_t	bufcnt;		/* byte count of allocated buffer */
	uint32_t        ccb_count;
	uint32_t        fail_count;
};

#ifdef	__cplusplus
}
#endif
#define L2CACHE_LINESIZE 64
#define P2ROUNDUP(SIZE, ALIGN) \
        (((SIZE) + ((ALIGN)-1)) & ~((ALIGN)-1))
#define ROUNDUP_CACHELINE(x)            P2ROUNDUP((x), L2CACHE_LINESIZE)

#endif /* _ASM */

#endif	/* _SYS_DAX_IMPL_H */
