#undef TRACE_SYSTEM
#define TRACE_SYSTEM rds

#if !defined(_TRACE_RDS_H) || defined(TRACE_HEADER_MULTI_READ)
#define _TRACE_RDS_H

#include <linux/tracepoint.h>

TRACE_EVENT(rds_sendmsg,

	TP_PROTO(u32 saddr, u32 daddr, u16 sport, u16 dport, u64 seq,
		 int ret, u64 dt),

	TP_ARGS(saddr, daddr, sport, dport, seq, ret, dt),

	TP_STRUCT__entry(
		__array(u8, laddr, 4)
		__array(u8, faddr, 4)
		__field(u16, sport)
		__field(u16, dport)
		__field(u64, seq)
		__field(u64, dt)
		__field(int, ret)
	),

	TP_fast_assign(
		memcpy(&__entry->laddr,  &saddr, 4);
		memcpy(&__entry->faddr,  &daddr, 4);
		__entry->sport = sport;
		__entry->dport = dport;
		__entry->seq = be64_to_cpu(seq);
		__entry->dt = dt;
		__entry->ret = ret;
	),

	TP_printk("local=%pI4/%d remote=%pI4/%d seq=%lld dt=%lld nsec ret=%d",
		  __entry->laddr, __entry->dport,
		  __entry->faddr, __entry->sport,
		  __entry->seq, __entry->dt, __entry->ret)
);

TRACE_EVENT(rds_sendmsg_rm_added,

	TP_PROTO(u32 saddr, u32 daddr, u16 sport, u16 dport, u64 seq),

	TP_ARGS(saddr, daddr, sport, dport, seq),

	TP_STRUCT__entry(
		__array(u8, laddr, 4)
		__array(u8, faddr, 4)
		__field(u16, sport)
		__field(u16, dport)
		__field(u64, seq)
	),

	TP_fast_assign(
		memcpy(&__entry->laddr,  &saddr, 4);
		memcpy(&__entry->faddr,  &daddr, 4);
		__entry->sport = sport;
		__entry->dport = dport;
		__entry->seq = be64_to_cpu(seq);
	),

	TP_printk("local=%pI4/%d remote=%pI4/%d seq=%lld",
		  __entry->laddr, __entry->sport,
		  __entry->faddr, __entry->dport,
		  __entry->seq)
);

TRACE_EVENT(rds_send_xmit,

	TP_PROTO(u32 laddr, u32 faddr, u64 dt, int ret),

	TP_ARGS(laddr, faddr, dt, ret),

	TP_STRUCT__entry(
		__array(u8, laddr, 4)
		__array(u8, faddr, 4)
		__field(u64, dt)
		__field(int, ret)
	),

	TP_fast_assign(
		memcpy(&__entry->laddr,  &laddr, 4);
		memcpy(&__entry->faddr,  &faddr, 4);
		__entry->dt  = dt/1000;
		__entry->ret = ret;
	),

	TP_printk("laddr=%pI4 faddr=%pI4 dt=%lld usec ret=%d",
		  __entry->laddr, __entry->faddr, __entry->dt, __entry->ret)
);

TRACE_EVENT(rds_recvmsg,

	TP_PROTO(u32 saddr, u32 daddr, u16 sport, u16 dport, u64 seq, u64 ack,
		 u64 dt, int ret),

	TP_ARGS(saddr, daddr, sport, dport, seq, ack, dt, ret),

	TP_STRUCT__entry(
		__array(u8, laddr, 4)
		__array(u8, faddr, 4)
		__field(u16, sport)
		__field(u16, dport)
		__field(u64, seq)
		__field(u64, ack)
		__field(u64, dt)
		__field(int, ret)
	),

	TP_fast_assign(
		memcpy(&__entry->faddr,  &saddr, 4);
		memcpy(&__entry->laddr,  &daddr, 4);
		__entry->sport = sport;
		__entry->dport = dport;
		__entry->seq = be64_to_cpu(seq);
		__entry->ack = be64_to_cpu(ack);
		__entry->dt = dt;
		__entry->ret = ret;
	),

	TP_printk("local=%pI4/%d remote=%pI4/%d seq=%lld ack=%lld dt=%lld nsec ret=%d",
		  __entry->laddr, __entry->dport,
		  __entry->faddr, __entry->sport,
		  __entry->seq, __entry->ack,
		  __entry->dt, __entry->ret)
);

TRACE_EVENT(rds_recv_incoming,

	TP_PROTO(u32 saddr, u32 daddr, u16 sport, u16 dport, u64 seq, u64 ack),

	TP_ARGS(saddr, daddr, sport, dport, seq, ack),

	TP_STRUCT__entry(
		__array(u8, laddr, 4)
		__array(u8, faddr, 4)
		__field(u16, sport)
		__field(u16, dport)
		__field(u64, seq)
		__field(u64, ack)
	),

	TP_fast_assign(
		memcpy(&__entry->laddr,  &daddr, 4);
		memcpy(&__entry->faddr,  &saddr, 4);
		__entry->sport = sport;
		__entry->dport = dport;
		__entry->seq = be64_to_cpu(seq);
		__entry->ack = be64_to_cpu(ack);
	),

	TP_printk("local=%pI4/%d remote=%pI4/%d seq=%lld ack=%lld",
		  __entry->laddr, __entry->dport,
		  __entry->faddr, __entry->sport,
		  __entry->seq, __entry->ack)
);

TRACE_EVENT(rds_send_worker,

	TP_PROTO(u32 laddr, u32 faddr, u64 dt),

	TP_ARGS(laddr, faddr, dt),

	TP_STRUCT__entry(
		__array(u8, laddr, 4)
		__array(u8, faddr, 4)
		__field(u64, dt)
	),

	TP_fast_assign(
		memcpy(&__entry->laddr,  &laddr, 4);
		memcpy(&__entry->faddr,  &faddr, 4);
		__entry->dt = dt/1000;
	),

	TP_printk("local=%pI4 remote=%pI4 dt=%lld usec",
		  __entry->laddr, __entry->faddr, __entry->dt)
);

TRACE_EVENT(rds_recv_worker,

	TP_PROTO(u32 laddr, u32 faddr, u64 dt),

	TP_ARGS(laddr, faddr, dt),

	TP_STRUCT__entry(
		__array(u8, laddr, 4)
		__array(u8, faddr, 4)
		__field(u64, dt)
	),

	TP_fast_assign(
		memcpy(&__entry->laddr,  &laddr, 4);
		memcpy(&__entry->faddr,  &faddr, 4);
		__entry->dt = dt/1000;
	),

	TP_printk("local=%pI4 remote=%pI4 dt=%lld usec",
		  __entry->laddr, __entry->faddr, __entry->dt)
);

TRACE_EVENT(rds_send_drop_acked,

	TP_PROTO(u32 laddr, u32 faddr, u64 dt),

	TP_ARGS(laddr, faddr, dt),

	TP_STRUCT__entry(
		__array(u8, laddr, 4)
		__array(u8, faddr, 4)
		__field(u64, dt)
	),

	TP_fast_assign(
		memcpy(&__entry->laddr,  &laddr, 4);
		memcpy(&__entry->faddr,  &faddr, 4);
		__entry->dt  = dt / 1000;
	),

	TP_printk("laddr=%pI4 faddr=%pI4 dt=%lld usec",
		  __entry->laddr, __entry->faddr, __entry->dt)
);

#endif /* _TRACE_RDS_H */

/* This part must be outside protection */
#include <trace/define_trace.h>
