#undef TRACE_SYSTEM
#define TRACE_SYSTEM rds_ib

#if !defined(_TRACE_RDS_IB_H) || defined(TRACE_HEADER_MULTI_READ)
#define _TRACE_RDS_IB_H

#include <linux/tracepoint.h>

DECLARE_EVENT_CLASS(rds_ib_conn_template,

	TP_PROTO(u32 laddr, u32 faddr, u64 dt),

	TP_ARGS(laddr, faddr, dt),

	TP_STRUCT__entry(
		__array(u8, laddr, 4)
		__array(u8, faddr, 4)
		__field(u64, dt)
	),

	TP_fast_assign(
		memcpy(&__entry->laddr,  &laddr, 4);
		memcpy(&__entry->faddr,  &faddr, 4);
		__entry->dt = dt/1000;
	),

	TP_printk("local=%pI4 remote=%pI4 dt=%lld usec",
		  __entry->laddr, __entry->faddr, __entry->dt)
);

DEFINE_EVENT(rds_ib_conn_template, rds_ib_tasklet_send,
	TP_PROTO(u32 laddr, u32 faddr, u64 dt),
	TP_ARGS(laddr, faddr, dt));

DEFINE_EVENT(rds_ib_conn_template, rds_ib_tasklet_recv,
	TP_PROTO(u32 laddr, u32 faddr, u64 dt),
	TP_ARGS(laddr, faddr, dt));

TRACE_EVENT(poll_cq,

	TP_PROTO(u32 laddr, u32 faddr, int nr, int rx, u64 dt),

	TP_ARGS(laddr, faddr, nr, rx, dt),

	TP_STRUCT__entry(
		__array(u8, laddr, 4)
		__array(u8, faddr, 4)
		__field(int, nr)
		__field(int, rx)
		__field(u64, dt)
	),

	TP_fast_assign(
		memcpy(&__entry->laddr,  &laddr, 4);
		memcpy(&__entry->faddr,  &faddr, 4);
		__entry->nr = nr;
		__entry->rx = rx;
		__entry->dt = dt / 1000;
	),

	TP_printk("local=%pI4 remote=%pI4 nr=%d rx=%d dt=%lld usec",
		  __entry->laddr, __entry->faddr, __entry->nr, __entry->rx,
		  __entry->dt)
);

TRACE_EVENT(rds_ib_xmit,

	TP_PROTO(u32 laddr, u32 faddr, u64 seq, int ret, u64 dt),

	TP_ARGS(laddr, faddr, seq, ret, dt),

	TP_STRUCT__entry(
		__array(u8, laddr, 4)
		__array(u8, faddr, 4)
		__field(u64, dt)
		__field(u64, seq)
		__field(int, ret)
	),

	TP_fast_assign(
		memcpy(&__entry->laddr,  &laddr, 4);
		memcpy(&__entry->faddr,  &faddr, 4);
		__entry->ret = ret;
		__entry->seq = be64_to_cpu(seq);
		__entry->dt = dt;
	),

	TP_printk("local=%pI4 remote=%pI4 seq=%lld dt=%lld ret=%d",
		  __entry->laddr, __entry->faddr, __entry->seq,
		  __entry->dt, __entry->ret)
);

TRACE_EVENT(rm_send_done,

	TP_PROTO(u32 laddr, u32 faddr, u64 seq),

	TP_ARGS(laddr, faddr, seq),

	TP_STRUCT__entry(
		__array(u8, laddr, 4)
		__array(u8, faddr, 4)
		__field(u64, seq)
	),

	TP_fast_assign(
		memcpy(&__entry->laddr,  &laddr, 4);
		memcpy(&__entry->faddr,  &faddr, 4);
		__entry->seq = be64_to_cpu(seq);
	),

	TP_printk("local=%pI4 remote=%pI4 seq=%lld",
		  __entry->laddr, __entry->faddr, __entry->seq)
);

TRACE_EVENT(rds_ib_flush_mr_pool,

	TP_PROTO(int free_all, unsigned int unpinned, unsigned int ncleaned,
		 unsigned int nfreed, u64 dt),

	TP_ARGS(free_all, unpinned, ncleaned, nfreed, dt),

	TP_STRUCT__entry(
		__field(int, free_all)
		__field(unsigned int, unpinned)
		__field(unsigned int, ncleaned)
		__field(unsigned int, nfreed)
		__field(u64, dt)
	),

	TP_fast_assign(
		__entry->free_all = free_all;
		__entry->unpinned = unpinned;
		__entry->ncleaned = ncleaned;
		__entry->nfreed   = nfreed;
		__entry->dt = dt / 1000;
	),

	TP_printk("free_all=%d unpinned=%d ncleaned=%d nfreed=%d dt=%lld usec",
		  __entry->free_all, __entry->unpinned, __entry->ncleaned,
		  __entry->nfreed, __entry->dt)
);

TRACE_EVENT(rds_ib_alloc_fmr,

	TP_PROTO(u64 dt, void *p),

	TP_ARGS(dt, p),

	TP_STRUCT__entry(
		__field(u64, dt)
		__field(unsigned long, ibmr)
	),

	TP_fast_assign(
		__entry->dt = dt;
		__entry->ibmr = (unsigned long) p;
	),

	TP_printk("ibmr=%lx dt=%lld nsec", __entry->ibmr, __entry->dt)
);

#endif /* _TRACE_RDS_IB_H */

/* This part must be outside protection */
#include <trace/define_trace.h>
